@@ -35,35 +35,34 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <string>
 #include <vector>
+#include <map>
+#include <memory>
 
 #include "libtorrent/aux_/disable_warnings_push.hpp"
-
-#include <boost/optional.hpp>
 #include <boost/shared_array.hpp>
-
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 #include "libtorrent/config.hpp"
+#include "libtorrent/fwd.hpp"
 #include "libtorrent/bdecode.hpp"
 #include "libtorrent/time.hpp"
 #include "libtorrent/assert.hpp"
 #include "libtorrent/copy_ptr.hpp"
-#include "libtorrent/socket.hpp"
 #include "libtorrent/sha1_hash.hpp"
 #include "libtorrent/file_storage.hpp"
+#include "libtorrent/aux_/vector.hpp"
+
+#if TORRENT_COMPLETE_TYPES_REQUIRED
+#include "libtorrent/announce_entry.hpp"
+#endif
+
+namespace libtorrent {
 
-namespace libtorrent
-{
-	class peer_connection;
-	class entry;
-	struct announce_entry;
 	struct lazy_entry;
 
-	namespace aux { struct session_settings; }
-
 	// internal, exposed for the unit test
 	TORRENT_EXTRA_EXPORT void sanitize_append_path_element(std::string& path
-		, char const* element, int element_len);
+		, string_view element);
 	TORRENT_EXTRA_EXPORT bool verify_encoding(std::string& target);
 
 	// the web_seed_entry holds information about a web seed (also known
@@ -76,7 +75,7 @@ namespace libtorrent
 		// http seed spec. by John Hoffman
 		enum type_t { url_seed, http_seed };
 
-		typedef std::vector<std::pair<std::string, std::string> > headers_t;
+		using headers_t = std::vector<std::pair<std::string, std::string>>;
 
 		web_seed_entry(std::string const& url_, type_t type_
 			, std::string const& auth_ = std::string()
@@ -84,7 +83,7 @@ namespace libtorrent
 
 		// URL and type comparison
 		bool operator==(web_seed_entry const& e) const
-		{ return url == e.url && type == e.type; }
+		{ return type == e.type && url == e.url; }
 
 		// URL and type less-than comparison
 		bool operator<(web_seed_entry const& e) const
@@ -106,13 +105,14 @@ namespace libtorrent
 		headers_t extra_headers;
 
 		// The type of web seed (see type_t)
-		boost::uint8_t type;
+		std::uint8_t type;
 	};
 
-#ifndef BOOST_NO_EXCEPTIONS
-	// for backwards compatibility with 0.14
-	typedef libtorrent_exception invalid_torrent_file;
-#endif
+	// hidden
+	class from_span_t {};
+
+	// used to disambiguate a bencoded buffer and a filename
+	extern TORRENT_EXPORT from_span_t from_span;
 
 	// TODO: there may be some opportunities to optimize the size if torrent_info.
 	// specifically to turn some std::string and std::vector into pointers
@@ -125,15 +125,15 @@ namespace libtorrent
 		// internally when downloading torrents without the metadata. The
 		// metadata will be created by libtorrent as soon as it has been
 		// downloaded from the swarm.
-		// 
+		//
 		// The constructor that takes a bdecode_node will create a torrent_info
 		// object from the information found in the given torrent_file. The
 		// bdecode_node represents a tree node in an bencoded file. To load an
 		// ordinary .torrent file into a bdecode_node, use bdecode().
-		// 
+		//
 		// The version that takes a buffer pointer and a size will decode it as a
 		// .torrent file and initialize the torrent_info object for you.
-		// 
+		//
 		// The version that takes a filename will simply load the torrent file
 		// and decode it inside the constructor, for convenience. This might not
 		// be the most suitable for applications that want to be able to report
@@ -142,41 +142,59 @@ namespace libtorrent
 		// There is an upper limit on the size of the torrent file that will be
 		// loaded by the overload taking a filename. If it's important that even
 		// very large torrent files are loaded, use one of the other overloads.
-		// 
+		//
 		// The overloads that takes an ``error_code const&`` never throws if an
 		// error occur, they will simply set the error code to describe what went
 		// wrong and not fully initialize the torrent_info object. The overloads
 		// that do not take the extra error_code parameter will always throw if
 		// an error occurs. These overloads are not available when building
 		// without exception support.
-		// 
-		// The ``flags`` argument is currently unused.
+		//
+		// The overload that takes a ``span`` also needs an extra parameter of
+		// type ``from_span_t`` to disambiguate the ``std::string`` overload for
+		// string literals. There is an object in the libtorrent namespace of this
+		// type called ``from_span``.
 #ifndef BOOST_NO_EXCEPTIONS
-		torrent_info(bdecode_node const& torrent_file, int flags = 0);
-		torrent_info(char const* buffer, int size, int flags = 0);
-		torrent_info(std::string const& filename, int flags = 0);
+		explicit torrent_info(bdecode_node const& torrent_file);
+		torrent_info(char const* buffer, int size)
+			: torrent_info(span<char const>{buffer, size}, from_span) {}
+		explicit torrent_info(span<char const> buffer, from_span_t);
+		explicit torrent_info(std::string const& filename);
 #endif // BOOST_NO_EXCEPTIONS
 		torrent_info(torrent_info const& t);
-		torrent_info(sha1_hash const& info_hash, int flags = 0);
-		torrent_info(bdecode_node const& torrent_file, error_code& ec, int flags = 0);
-		torrent_info(char const* buffer, int size, error_code& ec, int flags = 0);
-		torrent_info(std::string const& filename, error_code& ec, int flags = 0);
-#ifndef TORRENT_NO_DEPRECATE
+		explicit torrent_info(sha1_hash const& info_hash);
+		torrent_info(bdecode_node const& torrent_file, error_code& ec);
+		torrent_info(char const* buffer, int size, error_code& ec)
+			: torrent_info(span<char const>{buffer, size}, ec, from_span) {}
+		torrent_info(span<char const> buffer, error_code& ec, from_span_t);
+		torrent_info(std::string const& filename, error_code& ec);
+
+#if TORRENT_ABI_VERSION == 1
+#ifndef BOOST_NO_EXCEPTIONS
 		TORRENT_DEPRECATED
-		torrent_info(lazy_entry const& torrent_file, int flags = 0);
+		torrent_info(char const* buffer, int size, int)
+			: torrent_info(span<char const>{buffer, size}, from_span) {}
+#endif
 		TORRENT_DEPRECATED
-		torrent_info(lazy_entry const& torrent_file, error_code& ec
-			, int flags = 0);
-#if TORRENT_USE_WSTRING
+		torrent_info(bdecode_node const& torrent_file, error_code& ec, int)
+			: torrent_info(torrent_file, ec) {}
+		TORRENT_DEPRECATED
+		torrent_info(std::string const& filename, error_code& ec, int)
+			: torrent_info(filename, ec) {}
+		TORRENT_DEPRECATED
+		torrent_info(char const* buffer, int size, error_code& ec, int)
+			: torrent_info(span<char const>{buffer, size}, ec, from_span) {}
+		TORRENT_DEPRECATED
+		explicit torrent_info(lazy_entry const& torrent_file);
+		TORRENT_DEPRECATED
+		torrent_info(lazy_entry const& torrent_file, error_code& eca);
 		// all wstring APIs are deprecated since 0.16.11 instead, use the wchar
 		// -> utf8 conversion functions and pass in utf8 strings
 		TORRENT_DEPRECATED
-		torrent_info(std::wstring const& filename, error_code& ec
-			, int flags = 0);
+		torrent_info(std::wstring const& filename, error_code& ec);
 		TORRENT_DEPRECATED
-		torrent_info(std::wstring const& filename, int flags = 0);
-#endif // TORRENT_USE_WSTRING
-#endif // TORRENT_NO_DEPRECATE
+		explicit torrent_info(std::wstring const& filename);
+#endif // TORRENT_ABI_VERSION
 
 		// frees all storage associated with this torrent_info object
 		~torrent_info();
@@ -187,12 +205,12 @@ namespace libtorrent
 		// having a torrent file. When renaming files in a storage, the storage
 		// needs to make its own copy of the file_storage in order to make its
 		// mapping differ from the one in the torrent file.
-		// 
+		//
 		// ``orig_files()`` returns the original (unmodified) file storage for
 		// this torrent. This is used by the web server connection, which needs
 		// to request files with the original names. Filename may be changed using
 		// ``torrent_info::rename_file()``.
-		// 
+		//
 		// For more information on the file_storage object, see the separate
 		// document on how to create torrents.
 		file_storage const& files() const { return m_files; }
@@ -205,45 +223,43 @@ namespace libtorrent
 		// Renames a the file with the specified index to the new name. The new
 		// filename is reflected by the ``file_storage`` returned by ``files()``
 		// but not by the one returned by ``orig_files()``.
-		// 
+		//
 		// If you want to rename the base name of the torrent (for a multi file
 		// torrent), you can copy the ``file_storage`` (see files() and
 		// orig_files() ), change the name, and then use `remap_files()`_.
-		// 
+		//
 		// The ``new_filename`` can both be a relative path, in which case the
 		// file name is relative to the ``save_path`` of the torrent. If the
 		// ``new_filename`` is an absolute path (i.e. ``is_complete(new_filename)
 		// == true``), then the file is detached from the ``save_path`` of the
 		// torrent. In this case the file is not moved when move_storage() is
 		// invoked.
-		void rename_file(int index, std::string const& new_filename)
+		void rename_file(file_index_t index, std::string const& new_filename)
 		{
 			TORRENT_ASSERT(is_loaded());
+			if (m_files.file_path(index) == new_filename) return;
 			copy_on_write();
 			m_files.rename_file(index, new_filename);
 		}
-#ifndef TORRENT_NO_DEPRECATE
-#if TORRENT_USE_WSTRING
+#if TORRENT_ABI_VERSION == 1
 		// all wstring APIs are deprecated since 0.16.11
 		// instead, use the wchar -> utf8 conversion functions
 		// and pass in utf8 strings
 		TORRENT_DEPRECATED
-		void rename_file(int index, std::wstring const& new_filename);
-#endif // TORRENT_USE_WSTRING
-#endif // TORRENT_NO_DEPRECATE
+		void rename_file(file_index_t index, std::wstring const& new_filename);
+#endif // TORRENT_ABI_VERSION
 
 		// Remaps the file storage to a new file layout. This can be used to, for
 		// instance, download all data in a torrent to a single file, or to a
 		// number of fixed size sector aligned files, regardless of the number
 		// and sizes of the files in the torrent.
-		// 
+		//
 		// The new specified ``file_storage`` must have the exact same size as
 		// the current one.
 		void remap_files(file_storage const& f);
 
 		// ``add_tracker()`` adds a tracker to the announce-list. The ``tier``
 		// determines the order in which the trackers are to be tried.
-		//
 		// The ``trackers()`` function will return a sorted vector of
 		// ``announce_entry``. Each announce entry contains a string, which is
 		// the tracker url, and a tier index. The tier index is the high-level
@@ -258,12 +274,12 @@ namespace libtorrent
 		// "collections" keys in the .torrent file. Both info-hashes and
 		// collections from within the info-dict and from outside of it are
 		// included.
-		// 
+		//
 		// .. _`BEP 38`: http://www.bittorrent.org/beps/bep_0038.html
 		std::vector<sha1_hash> similar_torrents() const;
 		std::vector<std::string> collections() const;
 
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 		// deprecated in 0.16. Use web_seeds() instead
 		TORRENT_DEPRECATED
 		std::vector<std::string> url_seeds() const;
@@ -272,30 +288,29 @@ namespace libtorrent
 
 		// deprecated in 1.1
 		TORRENT_DEPRECATED
-		bool parse_info_section(lazy_entry const& e, error_code& ec
-			, int flags);
-#endif // TORRENT_NO_DEPRECATE
+		bool parse_info_section(lazy_entry const& e, error_code& ec);
+#endif // TORRENT_ABI_VERSION
 
 		// ``web_seeds()`` returns all url seeds and http seeds in the torrent.
 		// Each entry is a ``web_seed_entry`` and may refer to either a url seed
 		// or http seed.
-		// 
+		//
 		// ``add_url_seed()`` and ``add_http_seed()`` adds one url to the list of
 		// url/http seeds. Currently, the only transport protocol supported for
 		// the url is http.
-		// 
+		//
 		// ``set_web_seeds()`` replaces all web seeds with the ones specified in
 		// the ``seeds`` vector.
-		// 
+		//
 		// The ``extern_auth`` argument can be used for other authorization
 		// schemes than basic HTTP authorization. If set, it will override any
 		// username and password found in the URL itself. The string will be sent
 		// as the HTTP authorization header's value (without specifying "Basic").
-		// 
+		//
 		// The ``extra_headers`` argument defaults to an empty list, but can be
 		// used to insert custom HTTP headers in the requests to a specific web
 		// seed.
-		// 
+		//
 		// See http-seeding_ for more information.
 		void add_url_seed(std::string const& url
 			, std::string const& extern_auth = std::string()
@@ -314,25 +329,40 @@ namespace libtorrent
 		// argument and gives you the exact size of that piece. It will always be
 		// the same as ``piece_length()`` except in the case of the last piece,
 		// which may be smaller.
-		boost::int64_t total_size() const { return m_files.total_size(); }
+		std::int64_t total_size() const { return m_files.total_size(); }
 		int piece_length() const { return m_files.piece_length(); }
 		int num_pieces() const { return m_files.num_pieces(); }
 
+		// ``last_piece()`` returns the index to the last piece in the torrent and
+		// ``end_piece()`` returns the index to the one-past-end piece in the
+		// torrent
+		// ``piece_range()`` returns an implementation-defined type that can be
+		// used as the container in a range-for loop. Where the values are the
+		// indices of all pieces in the file_storage.
+		piece_index_t last_piece() const { return m_files.last_piece(); }
+		piece_index_t end_piece() const
+		{
+			TORRENT_ASSERT(m_files.num_pieces() > 0);
+			return m_files.end_piece();
+		}
+		index_range<piece_index_t> piece_range() const
+		{ return m_files.piece_range(); }
+
 		// returns the info-hash of the torrent
 		const sha1_hash& info_hash() const { return m_info_hash; }
 
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 		// deprecated in 1.0. Use the variants that take an index instead
 		// internal_file_entry is no longer exposed in the API
-		typedef file_storage::iterator file_iterator;
-		typedef file_storage::reverse_iterator reverse_file_iterator;
+		using file_iterator = file_storage::iterator;
+		using reverse_file_iterator = file_storage::reverse_iterator;
 
 		// This class will need some explanation. First of all, to get a list of
 		// all files in the torrent, you can use ``begin_files()``,
 		// ``end_files()``, ``rbegin_files()`` and ``rend_files()``. These will
 		// give you standard vector iterators with the type
 		// ``internal_file_entry``, which is an internal type.
-		// 
+		//
 		// You can resolve it into the public representation of a file
 		// (``file_entry``) using the ``file_storage::at`` function, which takes
 		// an index and an iterator.
@@ -345,12 +375,12 @@ namespace libtorrent
 		reverse_file_iterator rend_files() const { return m_files.rend_deprecated(); }
 
 		TORRENT_DEPRECATED
-		file_iterator file_at_offset(boost::int64_t offset) const
+		file_iterator file_at_offset(std::int64_t offset) const
 		{ return m_files.file_at_offset_deprecated(offset); }
 
 		TORRENT_DEPRECATED
 		file_entry file_at(int index) const { return m_files.at_deprecated(index); }
-#endif // TORRENT_NO_DEPRECATE
+#endif // TORRENT_ABI_VERSION
 
 		// If you need index-access to files you can use the ``num_files()`` along
 		// with the ``file_path()``, ``file_size()``-family of functions to access
@@ -360,7 +390,8 @@ namespace libtorrent
 		// This function will map a piece index, a byte offset within that piece
 		// and a size (in bytes) into the corresponding files with offsets where
 		// that data for that piece is supposed to be stored. See file_slice.
-		std::vector<file_slice> map_block(int piece, boost::int64_t offset, int size) const
+		std::vector<file_slice> map_block(piece_index_t const piece
+			, std::int64_t offset, int size) const
 		{
 			TORRENT_ASSERT(is_loaded());
 			return m_files.map_block(piece, offset, size);
@@ -369,75 +400,76 @@ namespace libtorrent
 		// This function will map a range in a specific file into a range in the
 		// torrent. The ``file_offset`` parameter is the offset in the file,
 		// given in bytes, where 0 is the start of the file. See peer_request.
-		// 
+		//
 		// The input range is assumed to be valid within the torrent.
 		// ``file_offset`` + ``size`` is not allowed to be greater than the file
 		// size. ``file_index`` must refer to a valid file, i.e. it cannot be >=
 		// ``num_files()``.
-		peer_request map_file(int file, boost::int64_t offset, int size) const
+		peer_request map_file(file_index_t const file, std::int64_t offset, int size) const
 		{
 			TORRENT_ASSERT(is_loaded());
 			return m_files.map_file(file, offset, size);
 		}
 
-		// load and unload this torrent info
-		void load(char const* buffer, int size, error_code& ec);
-		void unload();
-
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 // ------- start deprecation -------
-// these functions will be removed in a future version
+		// deprecated in 1.2
+		void load(char const*, int, error_code&) {}
+		void unload() {}
+
 		TORRENT_DEPRECATED
-		torrent_info(entry const& torrent_file);
-		TORRENT_DEPRECATED
-		void print(std::ostream& os) const;
+		explicit torrent_info(entry const& torrent_file);
 // ------- end deprecation -------
 #endif
 
 		// Returns the SSL root certificate for the torrent, if it is an SSL
 		// torrent. Otherwise returns an empty string. The certificate is
-		// the the public certificate in x509 format.
-		std::string ssl_cert() const;
+		// the public certificate in x509 format.
+		string_view ssl_cert() const;
 
 		// returns true if this torrent_info object has a torrent loaded.
 		// This is primarily used to determine if a magnet link has had its
 		// metadata resolved yet or not.
 		bool is_valid() const { return m_files.is_valid(); }
 
-		// returns true if this torrent is private. i.e., it should not be
-		// distributed on the trackerless network (the kademlia DHT).
-		bool priv() const { return m_private; }
+		// returns true if this torrent is private. i.e., the client should not
+		// advertise itself on the trackerless network (the Kademlia DHT) for this torrent.
+		bool priv() const { return (m_flags & private_torrent) != 0; }
 
 		// returns true if this is an i2p torrent. This is determined by whether
 		// or not it has a tracker whose URL domain name ends with ".i2p". i2p
 		// torrents disable the DHT and local peer discovery as well as talking
 		// to peers over anything other than the i2p network.
-		bool is_i2p() const { return m_i2p; }
+		bool is_i2p() const { return (m_flags & i2p) != 0; }
+
+		// returns the piece size of file with ``index``. This will be the same as piece_length(),
+		// except for the last piece, which may be shorter.
+		int piece_size(piece_index_t index) const { return m_files.piece_size(index); }
 
 		// ``hash_for_piece()`` takes a piece-index and returns the 20-bytes
 		// sha1-hash for that piece and ``info_hash()`` returns the 20-bytes
 		// sha1-hash for the info-section of the torrent file.
 		// ``hash_for_piece_ptr()`` returns a pointer to the 20 byte sha1 digest
-		// for the piece. Note that the string is not null-terminated.
-		int piece_size(int index) const { return m_files.piece_size(index); }
-		sha1_hash hash_for_piece(int index) const;
-		char const* hash_for_piece_ptr(int index) const
+		// for the piece. Note that the string is not 0-terminated.
+		sha1_hash hash_for_piece(piece_index_t index) const;
+		char const* hash_for_piece_ptr(piece_index_t const index) const
 		{
-			TORRENT_ASSERT(index >= 0);
-			TORRENT_ASSERT(index < m_files.num_pieces());
+			TORRENT_ASSERT(index >= piece_index_t(0));
+			TORRENT_ASSERT(index < m_files.end_piece());
 			TORRENT_ASSERT(is_loaded());
+			int const idx = static_cast<int>(index);
 			if (is_merkle_torrent())
 			{
-				TORRENT_ASSERT(index < int(m_merkle_tree.size() - m_merkle_first_leaf));
-				return m_merkle_tree[m_merkle_first_leaf + index].data();
+				TORRENT_ASSERT(idx < m_merkle_tree.end_index() - m_merkle_first_leaf);
+				return m_merkle_tree[m_merkle_first_leaf + idx].data();
 			}
 			else
 			{
 				TORRENT_ASSERT(m_piece_hashes);
 				TORRENT_ASSERT(m_piece_hashes >= m_info_section.get());
 				TORRENT_ASSERT(m_piece_hashes < m_info_section.get() + m_info_section_size);
-				TORRENT_ASSERT(index < int(m_info_section_size / 20));
-				return &m_piece_hashes[index*20];
+				TORRENT_ASSERT(idx < int(m_info_section_size / 20));
+				return &m_piece_hashes[idx * 20];
 			}
 		}
 
@@ -445,7 +477,6 @@ namespace libtorrent
 
 		// ``merkle_tree()`` returns a reference to the merkle tree for this
 		// torrent, if any.
-		// 
 		// ``set_merkle_tree()`` moves the passed in merkle tree into the
 		// torrent_info object. i.e. ``h`` will not be identical after the call.
 		// You need to set the merkle tree for a torrent that you've just created
@@ -458,32 +489,30 @@ namespace libtorrent
 		{ TORRENT_ASSERT(h.size() == m_merkle_tree.size() ); m_merkle_tree.swap(h); }
 
 		// ``name()`` returns the name of the torrent.
-		// 
-		// ``comment()`` returns the comment associated with the torrent. If
-		// there's no comment, it will return an empty string.
+		// name contains UTF-8 encoded string.
+		const std::string& name() const { return m_files.name(); }
+
 		// ``creation_date()`` returns the creation date of the torrent as time_t
 		// (`posix time`_). If there's no time stamp in the torrent file, the
 		// optional object will be uninitialized.
-		// 
-		// Both the name and the comment is UTF-8 encoded strings.
-		// 
+		// .. _`posix time`: http://www.opengroup.org/onlinepubs/009695399/functions/time.html
+		std::time_t creation_date() const
+		{ return m_creation_date; }
+
 		// ``creator()`` returns the creator string in the torrent. If there is
 		// no creator string it will return an empty string.
-		// 
-		// .. _`posix time`: http://www.opengroup.org/onlinepubs/009695399/functions/time.html
-		const std::string& name() const { return m_files.name(); }
-		boost::optional<time_t> creation_date() const;
 		const std::string& creator() const
 		{ return m_created_by; }
+
+		// ``comment()`` returns the comment associated with the torrent. If
+		// there's no comment, it will return an empty string.
+		// comment contains UTF-8 encoded string.
 		const std::string& comment() const
 		{ return m_comment; }
 
-		// dht nodes to add to the routing table/bootstrap from
-		typedef std::vector<std::pair<std::string, int> > nodes_t;
-
 		// If this torrent contains any DHT nodes, they are put in this vector in
 		// their original form (host name and port number).
-		nodes_t const& nodes() const
+		std::vector<std::pair<std::string, int>> const& nodes() const
 		{ return m_nodes; }
 
 		// This is used when creating torrent. Use this to add a known DHT node.
@@ -495,13 +524,12 @@ namespace libtorrent
 		// This is used when loading a torrent from a magnet link for instance,
 		// where we only have the info-dict. The bdecode_node ``e`` points to a
 		// parsed info-dictionary. ``ec`` returns an error code if something
-		// fails (typically if the info dictionary is malformed). ``flags`` are
-		// currently unused.
-		bool parse_info_section(bdecode_node const& e, error_code& ec, int flags);
+		// fails (typically if the info dictionary is malformed).
+		bool parse_info_section(bdecode_node const& e, error_code& ec);
 
 		// This function looks up keys from the info-dictionary of the loaded
 		// torrent file. It can be used to access extension values put in the
-		// .torrent file. If the specified key cannot be found, it returns NULL.
+		// .torrent file. If the specified key cannot be found, it returns nullptr.
 		bdecode_node info(char const* key) const;
 
 		// swap the content of this and ``ti``.
@@ -515,8 +543,8 @@ namespace libtorrent
 
 		// internal
 		bool add_merkle_nodes(std::map<int, sha1_hash> const& subtree
-			, int piece);
-		std::map<int, sha1_hash> build_merkle_list(int piece) const;
+			, piece_index_t piece);
+		std::map<int, sha1_hash> build_merkle_list(piece_index_t piece) const;
 
 		// returns whether or not this is a merkle torrent.
 		// see `BEP 30`__.
@@ -524,7 +552,7 @@ namespace libtorrent
 		// __ http://bittorrent.org/beps/bep_0030.html
 		bool is_merkle_torrent() const { return !m_merkle_tree.empty(); }
 
-		bool parse_torrent_file(bdecode_node const& libtorrent, error_code& ec, int flags);
+		bool parse_torrent_file(bdecode_node const& libtorrent, error_code& ec);
 
 		// if we're logging member offsets, we need access to them
 	private:
@@ -549,12 +577,15 @@ namespace libtorrent
 		// if m_files is modified, it is first copied into
 		// m_orig_files so that the original name and
 		// filenames are preserved.
+		// the original filenames are required to build URLs for web seeds for
+		// instance
 		copy_ptr<const file_storage> m_orig_files;
 
 		// the URLs to the trackers
-		std::vector<announce_entry> m_urls;
+		aux::vector<announce_entry> m_urls;
 		std::vector<web_seed_entry> m_web_seeds;
-		nodes_t m_nodes;
+		// dht nodes to add to the routing table/bootstrap from
+		std::vector<std::pair<std::string, int>> m_nodes;
 
 		// the info-hashes (20 bytes each) in the "similar" key. The pointers
 		// point directly into the info_section. When copied, these pointers must
@@ -569,8 +600,8 @@ namespace libtorrent
 		// these or strings of the "collections" key from the torrent file. The
 		// pointers point directly into the info_section buffer and when copied,
 		// these pointers must be corrected to point into the new buffer. The
-		// int is the length of the string. Strings are not NULL-terminated.
-		std::vector<std::pair<char const*, int> > m_collections;
+		// int is the length of the string. Strings are not 0-terminated.
+		std::vector<std::pair<char const*, int>> m_collections;
 
 		// these are the collections from outside of the info-dict. These are
 		// owning strings, since we only keep the info-section around, these
@@ -580,16 +611,17 @@ namespace libtorrent
 		// if this is a merkle torrent, this is the merkle
 		// tree. It has space for merkle_num_nodes(merkle_num_leafs(num_pieces))
 		// hashes
-		std::vector<sha1_hash> m_merkle_tree;
+		aux::vector<sha1_hash> m_merkle_tree;
 
 		// this is a copy of the info section from the torrent.
 		// it use maintained in this flat format in order to
 		// make it available through the metadata extension
+		// TODO: change the type to std::shared_ptr in C++17
 		boost::shared_array<char> m_info_section;
 
 		// this is a pointer into the m_info_section buffer
 		// pointing to the first byte of the first SHA-1 hash
-		char const* m_piece_hashes;
+		char const* m_piece_hashes = nullptr;
 
 		// if a comment is found in the torrent file
 		// this will be set to that comment
@@ -606,37 +638,46 @@ namespace libtorrent
 		// if a creation date is found in the torrent file
 		// this will be set to that, otherwise it'll be
 		// 1970, Jan 1
-		time_t m_creation_date;
+		std::time_t m_creation_date = 0;
 
 		// the hash that identifies this torrent
 		sha1_hash m_info_hash;
 
 		// the number of bytes in m_info_section
-		boost::uint32_t m_info_section_size;
+		std::int32_t m_info_section_size = 0;
 
 		// the index to the first leaf. This is where the hash for the
 		// first piece is stored
-		boost::uint32_t m_merkle_first_leaf:24;
+		std::int32_t m_merkle_first_leaf = 0;
 
-		// this is used when creating a torrent. If there's
-		// only one file there are cases where it's impossible
-		// to know if it should be written as a multi file torrent
-		// or not. e.g. test/test  there's one file and one directory
-		// and they have the same name.
-		bool m_multifile:1;
+		enum flags_t : std::uint8_t
+		{
+			// this is used when creating a torrent. If there's
+			// only one file there are cases where it's impossible
+			// to know if it should be written as a multi file torrent
+			// or not. e.g. test/test  there's one file and one directory
+			// and they have the same name.
+			multifile = 1,
 
-		// this is true if the torrent is private. i.e., is should not
-		// be announced on the dht
-		bool m_private:1;
+			// this is true if the torrent is private. i.e., is should not
+			// be announced on the dht
+			private_torrent = 2,
 
-		// this is true if one of the trackers has an .i2p top
-		// domain in its hostname. This means the DHT and LSD
-		// features are disabled for this torrent (unless the
-		// settings allows mixing i2p peers with regular peers)
-		bool m_i2p:1;
+			// this is true if one of the trackers has an .i2p top
+			// domain in its hostname. This means the DHT and LSD
+			// features are disabled for this torrent (unless the
+			// settings allows mixing i2p peers with regular peers)
+			i2p = 4,
+
+			// this flag is set if we found an ssl-cert field in the info
+			// dictionary
+			ssl_torrent = 8,
+		};
+
+		// any combination of values from flags_t enum
+		std::uint8_t m_flags = 0;
 	};
 
 }
 
 #endif // TORRENT_TORRENT_INFO_HPP_INCLUDED
-
